package Data::Sync::Shared::Semaphore;
use strict;
use warnings;
use Data::Sync::Shared ();
1;

__END__

=head1 NAME

Data::Sync::Shared::Semaphore - CAS-based counting semaphore

=head1 DESCRIPTION

See L<Data::Sync::Shared>.

=cut
